/*
STATA source code (dated 22 Jun 2023)
Paper: Modelling Bland-Altman Limits of Agreement with Fractional Polynomials -- An Example with the Agatston Score for Coronary Calcification
Authors: Oke Gerke & Sören Möller
Journal: Axioms (MDPI) 2023
*/
cd "c:\temp"
use "File_S2__interrater.dta", clear

/*inspection of paired differences*/
gen mean = (ct_agatston_score + cac_valid_2) / 2
gen diff =  ct_agatston_score - cac_valid_2 

hist diff, normal
stem diff

tab diff
tab study

tabstat diff, statistics(N mean sd min p5 p10 q p90 p95 max) by(study)

sort study diff
list 

*concord  ct_agatston_score cac_valid_2, loa(regline)


/*number of paired differences that deviated at least 50 HU from each other*/
gen absdiff= abs(diff)
sort diff
list study record_id ct_agatston_score cac_valid_2 mean diff if absdiff>50

gen ind50plus = cond(absdiff>50, 1, 0)
tab ind50plus


/*************************************************/
/*** Figure 2: scatter of differences vs means ***/
/*************************************************/
sort mean
twoway (scatter diff mean), xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-700(100)700, angle(0)) legend(off) 
graph export "Fig_2__2023_03_29.tif", as(tif) name("Graph") replace
graph export "Fig_2__2023_03_29.png", as(png) replace


/**************************************************/
/*** Fig. 3, right (Suppl. Fig. G (Gerke 2021)) ***/
/**************************************************/
scatter diff mean, xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-500(100)700, angle(0)) yline(0, lcolor(black) lpattern(dash)) yline(-82.45 37.70, lcolor(black) lpattern(solid)) 

graph export "Suppl_Fig_interrater__2021_05_07.tif", as(tif) name("Graph") replace
graph export "Suppl_Fig_interrater__2021_05_07.png", as(png) replace
graph save "Graph" "Suppl_Fig_interrater__2021_05_07.gph", replace

/*coverage nonparametric Limits of Agreement*/
gen out0 = cond(diff<-82.45 | diff>37.70, 1, 0)
tab out0


/****************************************/
/*** Fig. 3, left (classical BA plot) ***/
/****************************************/
sum diff
/*Bland-Altman Limits of Agreement*/
di -.9313044 - invnormal(.975)*80.2513 
di -.9313044 + invnormal(.975)*80.2513 

/*coverage Bland-Altman Limits of Agreement*/
gen out_clasBA = cond(diff<-158.221 | diff>156.358, 1, 0)
tab out_clasBA

sort mean
twoway (scatter diff mean), xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-500(100)700, angle(0)) legend(off) yline(-158.221 156.358, lcolor(red) lpattern(solid)) yline(-.9313044, lcolor(red) lpattern(dash))
graph export "Class_BA_Fig_interrater__2023_03_29.tif", as(tif) name("Graph") replace
graph export "Class_BA_Fig_interrater__2023_03_29.png", as(png) replace
graph save "Graph" "Class_BA_Fig_interrater__2023_03_29.gph", replace

save tempdat.dta, replace


/**********************************************/
/*** Fractional Polynomial Regression (m=2) ***/
/*** Eq. (6), Figure 5, top right           ***/
/**********************************************/
use tempdat.dta, clear
sum mean diff
/*using FPR the continuous outcome must be positive, CACS is nonnegative*/
gen mean1 = mean+.01

fp <mean1>, dimension(2): regress absdiff <mean1>
fp_select, alpha(0.05)

gen sd2 = (1.029373*mean1_1 + .0000143 *mean1_2 + .6248949) * sqrt(_pi/2)
gen repcoeff2_upp = 1.96*sd2
gen repcoeff2_low =-1.96*sd2

sort mean
twoway (scatter diff mean) (line repcoeff2_upp mean, lcolor(green)) (line repcoeff2_low mean, lcolor(green)), xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-700(200)700, angle(0)) legend(off) yline(0, lcolor(green) lpattern(dash))
graph export "Re-analysis_Fig_interrater_m2_2023_03_30.png", as(png) replace
graph save "Graph" "Re-analysis_Fig_interrater_m2_2023_03_30.gph", replace

gen out2 = cond(repcoeff2_upp<diff | repcoeff2_low>diff, 1, 0)
tab out2

li ct_agatston_score cac_valid_2 mean diff mean1 repcoeff2_upp repcoeff2_low if out2==1


/**********************************************/
/*** Fractional Polynomial Regression (m=1) ***/
/*** Eq. (9), Figure 5, top left            ***/
/**********************************************/
regress absdiff mean
gen sd1 = (.0628848*mean + .6730657 ) * sqrt(_pi/2)
gen repcoeff1_upp = 1.96*sd1
gen repcoeff1_low =-1.96*sd1

sort mean
twoway (scatter diff mean) (line repcoeff1_upp mean, lcolor(green)) (line repcoeff1_low mean, lcolor(green)), xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-700(200)700, angle(0)) legend(off) yline(0, lcolor(green) lpattern(dash))
graph export "Re-analysis_Fig_interrater_m1_2023_03_30.png", as(png) replace
graph save "Graph" "Re-analysis_Fig_interrater_m1_2023_03_30.gph", replace

gen out1 = cond(repcoeff1_upp<diff | repcoeff1_low>diff, 1, 0)
tab out1


/**********************************************/
/*** Fractional Polynomial Regression (m=3) ***/
/*** Figure 5, bottom, left                 ***/
/**********************************************/
use tempdat.dta, clear
sum mean diff
gen mean1 = mean+.01

fp <mean1>, dimension(3): regress absdiff <mean1>
fp_select, alpha(0.05)

gen sd3 = (1.019461 *mean1_1 + 6.64e-08*mean1_2 + -7.62e-09*mean1_3 + .8186861) * sqrt(_pi/2)
gen repcoeff3_upp = 1.96*sd3
gen repcoeff3_low =-1.96*sd3

sort mean
twoway (scatter diff mean) (line repcoeff3_upp mean, lcolor(green)) (line repcoeff3_low mean, lcolor(green)), xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-700(200)700, angle(0)) legend(off) yline(0, lcolor(green) lpattern(dash))
graph export "Re-analysis_Fig_interrater__2023_03_17.tif", as(tif) name("Graph") replace
graph export "Re-analysis_Fig_interrater_m3_2023_03_30.png", as(png) replace
graph save "Graph" "Re-analysis_Fig_interrater_m3_2023_03_30.gph", replace

gen out3 = cond(repcoeff3_upp<diff | repcoeff3_low>diff, 1, 0)
tab out3

li ct_agatston_score cac_valid_2 mean diff mean1 repcoeff3_upp repcoeff3_low if out3==1


/********************************************************/
/*** Square root-transformation (as in Sevrukov 2005) ***/
/*** Eq. (10), Figure 5, bottom right                 ***/
/********************************************************/
use tempdat.dta, clear
sum mean diff
gen sqrtmean = sqrt(mean)

regress absdiff sqrtmean, noconst

gen sd_sr = (2.193837*sqrtmean) * sqrt(_pi/2)
gen repcoeff_sr_upp = 1.96*sd_sr
gen repcoeff_sr_low =-1.96*sd_sr

sort mean
twoway (scatter diff mean) (line repcoeff_sr_upp mean, lcolor(green)) (line repcoeff_sr_low mean, lcolor(green)), xtitle("Interrater means [HU]") ytitle("Interrater differences [HU]") xlabel(0(500)4000, angle(45)) ylabel(-700(200)700, angle(0)) legend(off) yline(0, lcolor(green) lpattern(dash))
graph export "Re-analysis_Fig_interrater_sr_2023_03_30.png", as(png) replace
graph save "Graph" "Re-analysis_Fig_interrater_sr_2023_03_30.gph", replace

gen out_sr = cond(repcoeff_sr_upp<diff | repcoeff_sr_low>diff, 1, 0)
tab out_sr

li ct_agatston_score cac_valid_2 mean diff mean repcoeff_sr_upp repcoeff_sr_low if out_sr==1



/**************/
/** Figure 3 **/
/**************/
graph combine "Class_BA_Fig_interrater__2023_03_29" "Suppl_Fig_interrater__2021_05_07.gph", row(1) 
graph export "Fig_3__2023_03_30.tif", as(tif) name("Graph") width(1200) replace


/**************/
/** Figure 5 **/
/**************/
graph combine "Re-analysis_Fig_interrater_m1_2023_03_30.gph" "Re-analysis_Fig_interrater_m2_2023_03_30.gph" "Re-analysis_Fig_interrater_m3_2023_03_30.gph" "Re-analysis_Fig_interrater_sr_2023_03_30.gph", row(2)
graph export "Fig_4__2023_03_30.tif", as(tif) name("Graph") width(1200) replace

erase tempdat.dta
/*** end-of-file ***/